# created by zhaotao on 2020-5-14
# UI Control Class
# -*- coding: UTF-8 -*-

from PySide2 import QtWidgets, QtCore, QtGui
from PySide2.QtCore import Qt
from pymel.core import *
import  pymel.core.general as GL
import pymel.core.datatypes as PD
import pymel.core.animation as PA
import json
import os
import maya.api.OpenMaya as om
import maya.api.OpenMayaAnim as oma
import maya.OpenMayaUI as omui
from shiboken2 import wrapInstance


import QuickSelectWidget
reload(QuickSelectWidget)
import QuickSelectCreateWindow
reload(QuickSelectCreateWindow)
import QuickSelectRuleButton
reload(QuickSelectRuleButton)
import QuickSelectModifyWindow
reload(QuickSelectModifyWindow)

def getMayaMainWindow():
    """
    get maya main window
    :return: ptr of maya main window as QMainWindow
    """
    win = omui.MQtUtil.mainWindow()
    ptr = wrapInstance(long(win), QtWidgets.QMainWindow)
    return ptr

class QuickSelectWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super(QuickSelectWindow, self).__init__(parent=getMayaMainWindow())
        self.widget = QtWidgets.QWidget()
        self.ui = QuickSelectWidget.Ui_Form()
        self.ui.setupUi(self.widget)
        self.setCentralWidget(self.widget)
        self.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.setWindowFlags(self.windowFlags() | Qt.Dialog)
        self.setWindowTitle("Quick Select")
        self.resize(400, 500)
        self.Data = None
        self.curtName = None
        self.buttons = []
        self.initMode()
        self.ui.createModeBtn.clicked.connect(self.onCreateMode)
        self.ui.modeCombox.currentTextChanged.connect(self.onModeComboxChange)
        self.ui.modeModifyBtn.clicked.connect(self.onModifyMode)

    def closeEvent(self,event):
        curPath = self.GetCurrentDir()
        ruleFile = curPath + '/rule.json'
        self.Data['CurrentMode'] = self.curtName
        jsonStr = json.dumps(self.Data, indent=2, sort_keys=True)
        with open(ruleFile, 'w') as outFile:
            outFile.write(jsonStr)
            outFile.close()

    def GetCurrentDir(self):
        cc = os.path.abspath(os.path.dirname(__file__))
        return cc.replace('\\', '/')

    def initMode(self):
        self.ui.modeModifyBtn.setEnabled(False)
        curPath = self.GetCurrentDir()
        ruleFile = curPath + '/rule.json'
        if not os.path.exists(ruleFile):
            return
        with open(ruleFile,'r') as inFile:
            data = json.load(inFile)
            self.Data = data
        self.curtName = self.Data['CurrentMode']
        self.initModeComBox()
        self.refreshButtons()

    def createFinishCallBack(self):
        curPath = self.GetCurrentDir()
        ruleFile = curPath + '/rule.json'
        if os.path.exists(ruleFile):
            with open(ruleFile, 'r') as inFile:
                data = json.load(inFile)
                self.Data = data
                if self.curtName == None or self.curtName == '':
                    self.curtName = self.Data['CurrentMode']
        self.initModeComBox()
        self.refreshButtons()

    def onCreateMode(self):
        w = QuickSelectCreateWindow.QuickSelectCreateWindow(self)
        w.setFinishCallBack(self.createFinishCallBack)
        w.show()

    def initModeComBox(self):
        if self.Data == None:
            return
        self.ui.modeCombox.clear()
        for k in self.Data['ModeData']:
            name = k['name']
            if self.ui.modeCombox.findText(name) < 0:
                self.ui.modeCombox.addItem(name)
        self.ui.modeCombox.setCurrentText(self.curtName)
        if self.curtName == '':
            self.curtName = self.ui.modeCombox.currentText()
        if self.ui.modeCombox.currentText() != '':
            self.ui.modeModifyBtn.setEnabled(True)
        else:
            self.ui.modeModifyBtn.setEnabled(False)

    def findItem(self,data,name):
        for k in data['ModeData']:
            if k['name'] == name:
                return k
        return False

    def clearButtons(self):
        for k in self.buttons:
            self.ui.gridLayout.removeWidget(k)
            k.setParent(None)
        self.buttons = []

    def refreshButtons(self):
        self.clearButtons()
        if self.Data == None:
            return
        item = self.findItem(self.Data,self.curtName)
        if not item:
            return
        row = 0
        col = 0
        mo = 2

        for n in item['list']:
            button = QuickSelectRuleButton.QuickSelectRuleButton()
            button.setRule(n['button'],n['rule'])
            self.ui.gridLayout.addWidget(button,row,col % mo)
            self.buttons.append(button)
            col += 1
            if col == mo:
                row += 1
                col = 0

    def onModeComboxChange(self,text):
        self.curtName = text
        self.refreshButtons()

    def onDelFinishCallBack(self,modeName):
        self.curtName = modeName
        self.initModeComBox()
        self.refreshButtons()

    def onDelDelCallBack(self,modeName):
        item = self.findItem(self.Data,modeName)
        if not item:
            return
        self.Data['ModeData'].remove(item)
        self.curtName = ''
        self.initModeComBox()
        self.refreshButtons()

    def onModifyMode(self):
        w = QuickSelectModifyWindow.QuickSelectModifyWindow(self)
        w.setFinishCallBack(self.onDelFinishCallBack)
        w.setDelCallBack(self.onDelDelCallBack)
        item = self.findItem(self.Data, self.curtName)
        w.setModifyData(item)
        w.show()